"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkManual = exports.check = exports.fetch = void 0;
const client_1 = require("./client");
const debug_1 = __importDefault(require("debug"));
const apollo_boost_1 = require("apollo-boost");
const forEach_1 = __importDefault(require("lodash/forEach"));
const assert_1 = __importDefault(require("assert"));
const debug = (0, debug_1.default)('materialized-path:check');
const SCHEMA = process.env.MIGRATIONS_SCHEMA || 'public';
const MP_TABLE = process.env.MIGRATIONS_MP_TABLE || 'mp_example__links__mp';
const GRAPH_TABLE = process.env.MIGRATIONS_GRAPH_TABLE || 'mp_example__links';
const ID_TYPE = process.env.MIGRATIONS_ID_TYPE_GQL || 'Int';
const fetch = (type_id, idType = ID_TYPE, GRAPH_TABLE, MP_TABLE) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b;
    const result = yield client_1.client.query({ query: (0, apollo_boost_1.gql) `query FETCH($type_id: ${idType}) {
    mp: ${MP_TABLE}(where: { item: { type_id: { _eq: $type_id } } }, order_by: {id: asc}) {
      id item_id path_item_depth path_item_id root_id position_id custom
      by_position(order_by: { path_item_depth: asc }) { id item_id path_item_depth path_item_id root_id position_id }
    }
    nodes: ${GRAPH_TABLE}(where: { type_id: { _eq: $type_id } }) { from_id id to_id type_id in { from_id id to_id type_id } out { from_id id to_id type_id } }
  }`, variables: { type_id } });
    return { nodes: ((_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.nodes) || [], mp: ((_b = result === null || result === void 0 ? void 0 : result.data) === null || _b === void 0 ? void 0 : _b.mp) || [] };
});
exports.fetch = fetch;
const check = (hash, type_id) => __awaiter(void 0, void 0, void 0, function* () {
    const n = {};
    (0, forEach_1.default)(hash, (value, key) => { n[value] = key; });
    const { nodes, mp } = yield (0, exports.fetch)(type_id, ID_TYPE, GRAPH_TABLE, MP_TABLE);
    debug('checking');
    let valid = true;
    const invalid = (...args) => {
        valid = false;
        debug(...args);
    };
    if (!nodes.length)
        invalid(`nodes not fetched`);
    const nodesChecked = {};
    const markersChecked = {};
    const checkNode = (node) => {
        var _a, _b, _c;
        if (nodesChecked[node.id])
            return;
        else
            nodesChecked[node.id] = true;
        const isLink = !!((node === null || node === void 0 ? void 0 : node.from_id) && (node === null || node === void 0 ? void 0 : node.to_id));
        const isRoot = isLink ? false : !((_a = node === null || node === void 0 ? void 0 : node.in) === null || _a === void 0 ? void 0 : _a.length);
        const markers = mp.filter((m) => m.item_id === node.id);
        const flows = mp.filter((m) => m.item_id === node.id && m.path_item_id === node.id);
        debug(`check #${n[node.id]} ${isLink ? 'link' : 'node'} in${(_b = node === null || node === void 0 ? void 0 : node.in) === null || _b === void 0 ? void 0 : _b.length} out${(_c = node === null || node === void 0 ? void 0 : node.out) === null || _c === void 0 ? void 0 : _c.length}`, flows.map((pos) => {
            return `${n[pos.root_id]} [${pos.by_position.map((m) => `${n[m.path_item_id]}`).join(',')}]`;
        }));
        for (let f = 0; f < flows.length; f++) {
            const flow = flows[f];
            const hasPositionsWithNotFlowId = mp.find(mp => mp.position_id === flow.position_id && mp.item_id !== flow.item_id);
            if (hasPositionsWithNotFlowId)
                invalid(`invalid positions position_id ${flow.position_id} must exists only for item_id ${flow.item_id} but exists for ${hasPositionsWithNotFlowId.item_id}`);
            const equalPositionIds = mp.filter(mp => mp.position_id === flow.position_id && mp.item_id === flow.item_id);
            if (equalPositionIds.length != flow.path_item_depth + 1)
                invalid(`invalid positions position_id ${flow.position_id} duplicates ${equalPositionIds.length} for flow.item_id ${flow.item_id} flow.path_item_depth ${flow.path_item_depth}`);
        }
        if (isRoot) {
            if (markers.length !== 1)
                invalid(`invalid node #${n[node.id]} root but markers.length = ${markers.length}`);
        }
        if (!markers.length)
            invalid(`invalid node #${n[node.id]} markers lost, markers.length = ${markers.length}`);
        flows.forEach((position) => {
            checkPosition(position);
        });
    };
    const checkPosition = (position) => {
        position.by_position.forEach((marker, i) => {
            markersChecked[marker.id] = true;
            if (marker.position_id != position.position_id)
                invalid(`invalid position ${n[position.root_id]} [${position.by_position.map((m) => n[m.path_item_id]).join(',')}] position_id not equal`);
            const node = nodes.find((n) => n.id === marker.path_item_id);
            if (!node)
                invalid(`invalid position ${n[position.root_id]} [${position.by_position.map((m) => n[m.path_item_id]).join(',')}] node lost #${n[marker.path_item_id]}`);
        });
    };
    nodes.forEach((node) => {
        checkNode(node);
    });
    mp.forEach((marker) => {
        if (!markersChecked[marker.id])
            invalid(`invalid marker #${marker.id} of node #${n[marker.item_id]} ${n[marker.root_id]}[...${n[marker.path_item_id]}...]`);
    });
    if (!valid)
        throw new Error('invalid');
    return { nodes, mp };
});
exports.check = check;
const checkManual = (input, type_id, GRAPH_TABLE, MP_TABLE) => __awaiter(void 0, void 0, void 0, function* () {
    const { nodes, mp } = yield (0, exports.fetch)(type_id, ID_TYPE, GRAPH_TABLE, MP_TABLE);
    const ns = [];
    const ls = [];
    const handledMP = {};
    for (let i = 0; i < nodes.length; i++) {
        const n = nodes[i];
        const l = input[i];
        ns.push({ id: n.id, from_id: n.from_id || 0, to_id: n.to_id || 0, type_id: n.type_id || 0, _by_item: mp.filter((mp) => {
                handledMP[mp.id] = true;
                return mp.item_id === n.id;
            }).map(a => ({ root_id: a.root_id, path_item_id: a.path_item_id })) });
        ls.push({ id: l[0], from_id: l[1], to_id: l[2], type_id: l[3], _by_item: l[4].map((a) => {
                return { root_id: a[0], path_item_id: a[1] };
            }) });
    }
    assert_1.default.deepStrictEqual(ns, ls);
    const notHandledMp = [];
    for (let i = 0; i < mp.length; i++) {
        const mark = mp[i];
        if (!handledMP[mark.id])
            notHandledMp[mark.id];
    }
    if (notHandledMp.length)
        throw new Error(`Not handled MP marks: ${notHandledMp}`);
});
exports.checkManual = checkManual;
//# sourceMappingURL=check.js.map