"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.down = exports.up = void 0;
const DEFAULT_SCHEMA = process.env.MIGRATIONS_SCHEMA || 'public';
const DEFAULT_MP_TABLE = process.env.MIGRATIONS_MP_TABLE || 'mp_example__links__mp';
const DEFAULT_GRAPH_TABLE = process.env.MIGRATIONS_GRAPH_TABLE || 'mp_example__links';
const up = ({ SCHEMA = DEFAULT_SCHEMA, MP_TABLE = DEFAULT_MP_TABLE, GRAPH_TABLE = DEFAULT_GRAPH_TABLE, ID_FIELD = 'id', api }) => __awaiter(void 0, void 0, void 0, function* () {
    yield api.query({
        type: 'create_select_permission',
        args: {
            table: MP_TABLE,
            role: 'guest',
            permission: {
                columns: '*',
                filter: {},
                limit: 999,
                allow_aggregations: true
            }
        }
    });
    yield api.query({
        type: 'create_select_permission',
        args: {
            table: MP_TABLE,
            role: 'user',
            permission: {
                columns: '*',
                filter: {},
                limit: 999,
                allow_aggregations: true
            }
        }
    });
    yield api.query({
        type: 'create_array_relationship',
        args: {
            table: GRAPH_TABLE,
            name: '_by_item',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: SCHEMA,
                        name: MP_TABLE,
                    },
                    column_mapping: {
                        [ID_FIELD]: 'item_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield api.query({
        type: 'create_array_relationship',
        args: {
            table: GRAPH_TABLE,
            name: '_by_path_item',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: SCHEMA,
                        name: MP_TABLE,
                    },
                    column_mapping: {
                        [ID_FIELD]: 'path_item_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield api.query({
        type: 'create_array_relationship',
        args: {
            table: GRAPH_TABLE,
            name: '_by_root',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: SCHEMA,
                        name: MP_TABLE,
                    },
                    column_mapping: {
                        [ID_FIELD]: 'root_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield api.query({
        type: 'create_array_relationship',
        args: {
            table: GRAPH_TABLE,
            name: '_by_group',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: SCHEMA,
                        name: MP_TABLE,
                    },
                    column_mapping: {
                        [ID_FIELD]: 'group_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield api.query({
        type: 'create_object_relationship',
        args: {
            table: MP_TABLE,
            name: 'item',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: SCHEMA,
                        name: GRAPH_TABLE,
                    },
                    column_mapping: {
                        item_id: ID_FIELD,
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield api.query({
        type: 'create_object_relationship',
        args: {
            table: MP_TABLE,
            name: 'path_item',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: SCHEMA,
                        name: GRAPH_TABLE,
                    },
                    column_mapping: {
                        path_item_id: ID_FIELD,
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield api.query({
        type: 'create_object_relationship',
        args: {
            table: MP_TABLE,
            name: 'root',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: SCHEMA,
                        name: GRAPH_TABLE,
                    },
                    column_mapping: {
                        root_id: ID_FIELD,
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield api.query({
        type: 'create_array_relationship',
        args: {
            table: MP_TABLE,
            name: 'by_item',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: SCHEMA,
                        name: MP_TABLE,
                    },
                    column_mapping: {
                        item_id: 'item_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield api.query({
        type: 'create_array_relationship',
        args: {
            table: MP_TABLE,
            name: 'by_path_item',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: SCHEMA,
                        name: MP_TABLE,
                    },
                    column_mapping: {
                        path_item_id: 'path_item_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield api.query({
        type: 'create_array_relationship',
        args: {
            table: MP_TABLE,
            name: 'by_position',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: SCHEMA,
                        name: MP_TABLE,
                    },
                    column_mapping: {
                        position_id: 'position_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield api.query({
        type: 'create_object_relationship',
        args: {
            table: MP_TABLE,
            name: 'by_group',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: SCHEMA,
                        name: GRAPH_TABLE,
                    },
                    column_mapping: {
                        group_id: ID_FIELD,
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
    yield api.query({
        type: 'create_array_relationship',
        args: {
            table: MP_TABLE,
            name: 'by_root',
            using: {
                manual_configuration: {
                    remote_table: {
                        schema: SCHEMA,
                        name: MP_TABLE,
                    },
                    column_mapping: {
                        root_id: 'root_id',
                    },
                    insertion_order: 'after_parent',
                },
            },
        },
    });
});
exports.up = up;
const down = ({ SCHEMA = DEFAULT_SCHEMA, MP_TABLE = DEFAULT_MP_TABLE, GRAPH_TABLE = DEFAULT_GRAPH_TABLE, api }) => __awaiter(void 0, void 0, void 0, function* () {
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: GRAPH_TABLE,
            relationship: '_by_item',
            cascade: true,
        },
    });
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: GRAPH_TABLE,
            relationship: '_by_path_item',
            cascade: true,
        },
    });
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: GRAPH_TABLE,
            relationship: '_by_root',
            cascade: true,
        },
    });
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: GRAPH_TABLE,
            relationship: '_by_group',
            cascade: true,
        },
    });
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: MP_TABLE,
            relationship: 'item',
            cascade: true,
        },
    });
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: MP_TABLE,
            relationship: 'path_item',
            cascade: true,
        },
    });
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: MP_TABLE,
            relationship: 'root',
            cascade: true,
        },
    });
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: MP_TABLE,
            relationship: 'by_item',
            cascade: true,
        },
    });
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: MP_TABLE,
            relationship: 'by_path_item',
            cascade: true,
        },
    });
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: MP_TABLE,
            relationship: 'by_position',
            cascade: true,
        },
    });
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: MP_TABLE,
            relationship: 'by_group',
            cascade: true,
        },
    });
    yield api.query({
        type: 'drop_relationship',
        args: {
            table: MP_TABLE,
            relationship: 'by_root',
            cascade: true,
        },
    });
});
exports.down = down;
//# sourceMappingURL=relationships.js.map